class V1::LearningObjectsComplaintsController < ApplicationController

    include ::DeletedObjectsController
    include ::Paginator

    # learning_object_complaints[i] = data of a complaint (always in even positions)
    # learning_object_complaints[i+1] = image of learning_object, and user, also contains the tags of the object and the download link (always in odd positions)

    def generate_priority_queue(complaints_priority)

        complaints_curator = []
        complaints_priority.each do |complainable|

            # Select all complaints related to the complainable_id (attributes that have been sumed)
            complaints_curator << ComplaintReason.joins("INNER JOIN complaints ON complaint_reasons.id = complaints.complaint_reason_id 
                 INNER JOIN learning_objects on complaints.complainable_id = learning_objects.id
                 INNER JOIN users ON learning_objects.publisher_id = users.id 
                 INNER JOIN object_types ON learning_objects.object_type_id = object_types.id
                 ")
                .select("learning_objects.id as learning_object_id, learning_objects.name, learning_objects.review_average, 
                    learning_objects.likes_count, users.name as publisher_name,
                    object_types.name as type_object, complaints.description, complaint_reasons.id as complaint_reason_id, 
                    complaint_reasons.reason, complaint_reasons.priority")
                .where("complaints.complainable_id = ? AND complaints.state = 0", complainable.complainable_id).order("complaint_reasons.priority DESC")

            #find the learning_object by the complainable_id
            learning_object = LearningObject.where(id: complainable.complainable_id)
            
            lo_data = Hash.new

            tags = Tagging.joins(:tag).where("taggings.taggable_id = ?", complainable.complainable_id).select("tags.name")
            all_tags = []

            #add all tags to lo_data hash
            tags.each do |tag|
                if (tag.name != nil)
                    all_tags << tag.name
                end
            end

            lo_data['tags'] = all_tags
            
            #iterate by all learning_objects found
            for lo in learning_object
                
                lo_data['learning_object_thumb'] = lo.thumbnail
                if (lo.publisher_id != nil) 
                    user = User.where(id: lo.publisher_id).first
                    lo_data['user_avatar'] = user.avatar
                end
                lo_data['download_link'] = lo.download_link

            end

            complaints_curator << lo_data
        end
        
        return complaints_curator
        
    end

    # GET v1/learning_objects_complaints
    def index
                
        
        #select and sum the priority if has two or more complaints
        complaints_priority = ComplaintReason.joins(:complaints).select("complainable_id, SUM(complaint_reasons.priority) as priorities")
        .where("complaints.state = 0").group("complainable_id").having("(COUNT(complainable_id) > 1 OR SUM(complaint_reasons.priority) > 20)")
        .order("priorities DESC").order("complainable_id").limit(params["limit"]).offset(params["offset"])
        
        total = ComplaintReason.joins(:complaints).select("complainable_id, SUM(complaint_reasons.priority)")
        .where("complaints.state = 0").group("complainable_id").having("COUNT(complainable_id) > 1 OR SUM(complaint_reasons.priority) > 20")

        queue = generate_priority_queue(complaints_priority)
        complaints_queue = paginate_select(queue, total.length)

        render json: complaints_queue

    end

    # GET v1/learning_objects_complaints/queue_user/:user_id
    def queue_user
        #select and sum the priority if has two or more complaints
        complaints_priority = ComplaintReason.joins(:complaints).select("complainable_id, SUM(complaint_reasons.priority) as priorities")
        .where("complaints.state = 0").where.not("complaints.user_id = ?",params[:user_id]).group("complainable_id").having("(COUNT(complainable_id) > 1 OR SUM(complaint_reasons.priority) > 20)")
        .order("priorities DESC").order("complainable_id").limit(params["limit"]).offset(params["offset"])
        
        total = ComplaintReason.joins(:complaints).select("complainable_id, SUM(complaint_reasons.priority)")
        .where("complaints.state = 0").where.not("complaints.user_id = ?",params[:user_id]).group("complainable_id").having("COUNT(complainable_id) > 1 OR SUM(complaint_reasons.priority) > 20")

        queue = generate_priority_queue(complaints_priority)
        complaints_queue = paginate_select(queue, total.length)

        render json: complaints_queue

    end
    
    def show
        id = params[:id]
        complaints = ComplaintReason.joins("INNER JOIN complaints ON complaint_reasons.id = complaints.complaint_reason_id")
        .where("complaints.complainable_id = ?",id)
        .select("complaints.description, complaint_reasons.reason, complaint_reasons.priority")
        render json: complaints
    end

end

